/*
 * app.c
 *
 *  Created on: Aug 12, 2020
 *      Author: kaige
 */

#include <stdio.h>
#include "cpu_core.h"
#include "ucos_ii.h"
#include "ucos_bsp.h"

#include "netif/xemacpsif.h"
#include "netif/xadapter.h"
#include "lwip/init.h"
#include "lwip/inet.h"
#include "lwip/opt.h"
#include "lwip/ip_addr.h"


#include "lwipopts.h"
#include <ucos_int.h>
#include "lwip/sockets.h"

#include "xipipsu.h"
#include "xscugic.h"

#include "netif/xemacps_ieee1588.h"
#include "ida-lwip/ida_lwip_init.h"
#include "ida-lwip/ida_lwip_filter.h"

#define UDP_SERVER_TASK_STACK_SIZE 1024
#define UDP_SERVER_TASK_PRIO 12

#define DUMMY_TASK_STACK_SIZE 512
#define DUMMY_TASK_PRIO 10

#define SOCK_SERVER_TASK_STACK_SIZE 128 // before optimization: 256 --> reduction by 25%
#define SOCK_SERVER_TASK_PRIO 15


static CPU_STK sockServerTaskStk[SOCK_SERVER_TASK_STACK_SIZE];



static void stopTest(void){
	while(1);;
}

static volatile void rxA(void){
	asm volatile("nop");
}

static volatile void rxB(void){
	asm volatile("nop");
}

static volatile void rxC(void){
	asm volatile("nop");
}

char rxBuffer[1500];
char objectDummy[1024];

void socketServerTask(void *p_arg){
	int socket_fds[10];					/* RX Sockets */
	int rxReady_fd = -1;				/* RX Socket Ready for read */
#ifdef IDA_LWIP
	int proxy_fd = -1;					/* Proxy Socket */
#else
	fd_set readfds;						/* Read FD Set */
#endif
	struct sockaddr_in rx_sockAddr;		/* Receive Address */
	int recv_data, i;

	printf("SOCKET RX SERVER TASK RUNNING\n");

	/* Create 10 sockets for reading */
	for(int i = 0; i < 10; i++){
		socket_fds[i] = -1;
		socket_fds[i] = socket(PF_INET, SOCK_DGRAM, 0);
		if(socket_fds[i] < 0)
			goto ERROR_EXIT;
	}

#ifdef IDA_LWIP
	/* Create Proxy Socket */
	proxy_fd = socket(PF_INET, SOCK_PROXY, 0);
	if(proxy_fd < 0)
		goto ERROR_EXIT;
#endif

	/* Bind all sockets to addresses 1000 - 1009 */
	memset(&rx_sockAddr, 0, sizeof(struct sockaddr_in));
	rx_sockAddr.sin_family = AF_INET;
	rx_sockAddr.sin_addr.s_addr = htonl(INADDR_ANY);

	for(int i = 0; i < 10; i++){
		rx_sockAddr.sin_port = htons(1000+i);
		if(bind(socket_fds[i], (struct sockaddr *)&rx_sockAddr, sizeof(struct sockaddr_in)) == -1)
			goto ERROR_EXIT;
	}

#ifdef IDA_LWIP
	/* Set socket priorities and register sockets to proxy socket */
	for(int i = 0; i < 10; i++){
		ida_lwip_set_socket_prio(socket_fds[i],i%4);

		sendto(proxy_fd, &socket_fds[i], sizeof(int), 0, NULL, 0);
	}
#endif

	i=0;
	while(1){
#ifdef IDA_LWIP
		/* Wait for proxy socket */
		if(recv(proxy_fd, &rxReady_fd, sizeof(rxReady_fd),0) != -1){
#else
		/* Register Read FD Set and call select */
		rxReady_fd = -1;
		FD_ZERO(&readfds);
		for(int i = 0; i < 10; i++){
			FD_SET(socket_fds[i], &readfds);
		}
		select(socket_fds[9] + 1 , &readfds , NULL , NULL , NULL);

		/* Get first socket with pending data */
		for(int i = 0; i < 10; i++){
			if (FD_ISSET(socket_fds[i], &readfds)){
				rxReady_fd = socket_fds[i];
				break;
			}
		}
		if(rxReady_fd >= 0) {
#endif

#if IDA_LWIP_PARTIAL_UDP_RX > 0 && defined(IDA_LWIP)
			rxA();
			/* Receive (MW-Header | Payload) */
			recv_data = recv(rxReady_fd,rxBuffer,6,0);
			if(recv_data > 0)
			{
				rxB();
				if(recv_data > 0) {
					/* Take header from payload */
					uint8_t seqNr = rxBuffer[0];
					/* Copy payload to application */
					recv_data = recv(rxReady_fd,objectDummy,1024,0);
					/* We are done */
					rxC();
				}
			}
#else
			rxA();
			/* Receive (MW-Header | Payload) */
			recv_data = recv(rxReady_fd,rxBuffer,sizeof(rxBuffer),0);
			rxB();
			if(recv_data > 0) {
				/* Take header from payload */
				uint8_t seqNr = rxBuffer[0];
				/* Copy payload to application */
				memcpy(objectDummy, &rxBuffer[2], 1024);
				/* We are done */
				rxC();
			}

#endif
			i++;
			if(i > 60000)
				stopTest();
		} else {
			goto ERROR_EXIT;
		}
	}

ERROR_EXIT:
	printf("Problem in socket server task\n");
	for(int i = 0; i < 10; i++)
		close(socket_fds[i]);
#ifdef IDA_LWIP
	close(proxy_fd);
#endif
	OSTaskDel(OS_PRIO_SELF);
}



void startAPP(void){


	OSTaskCreateExt(socketServerTask,
						NULL,
						&sockServerTaskStk[SOCK_SERVER_TASK_STACK_SIZE - 1],
						SOCK_SERVER_TASK_PRIO,
						SOCK_SERVER_TASK_PRIO,
						&sockServerTaskStk[0],
						SOCK_SERVER_TASK_STACK_SIZE,
						DEF_NULL,
						(OS_TASK_OPT_STK_CLR | OS_TASK_OPT_STK_CHK));
	OSTaskNameSet(SOCK_SERVER_TASK_PRIO, (INT8U *) "sockServerTask", NULL);

	return;
}
