/*
*********************************************************************************************************
*                                            EXAMPLE CODE
*
*               This file is provided as an example on how to use Micrium products.
*
*               Please feel free to use any application code labeled as 'EXAMPLE CODE' in
*               your application products.  Example code may be used as is, in whole or in
*               part, or may be used as a reference only. This file can be modified as
*               required to meet the end-product requirements.
*
*               Please help us continue to provide the Embedded community with the finest
*               software available.  Your honesty is greatly appreciated.
*
*               You can find our product's user manual, API reference, release notes and
*               more information at https://doc.micrium.com.
*               You can contact us at www.micrium.com.
*********************************************************************************************************
*/

/*
*********************************************************************************************************
*
*                                             uC/OS-II
*                                         Application Hooks
*
* Filename      : app_hooks.c
* Version       : V1.00
* Programmer(s) : FT
*********************************************************************************************************
*/

/*
*********************************************************************************************************
*                                             INCLUDE FILES
*********************************************************************************************************
*/

#include  <ucos_ii.h>
#include "imw_types.h"
#include "ILM/if/ilm_hooks.h"
#include "ptpd.h"
#include "imw_example_configs.h"
#include "imw_header.h"
#include "ida-lwip/ida_lwip_filter.h"
#include "netif/xemacps_ieee1588.h"

/*
*********************************************************************************************************
*                                       EXTERN  GLOBAL VARIABLES
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*                                           LOCAL CONSTANTS
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*                                          LOCAL DATA TYPES
*********************************************************************************************************
*/

/*
*********************************************************************************************************
*                                            LOCAL TABLES
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*                                       LOCAL GLOBAL VARIABLES
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*                                      LOCAL FUNCTION PROTOTYPES
*********************************************************************************************************
*/



/*
**********************************************************************************************************
**********************************************************************************************************
**                                        GLOBAL FUNCTIONS
**********************************************************************************************************
**********************************************************************************************************
*/

volatile __attribute__ ((noinline)) void stopTest(){
	asm volatile("nop");
}

const volatile void* pStimulusBase = (void*)0xf8000000;

static volatile uint32_t hookName = 0;
static volatile int running __attribute__ ((unused)) = 0;
static volatile uint32_t hookParameter1 = 0;
static volatile uint32_t hookParameter2 = 0;
static volatile uint32_t hookParameter3 = 0;
static volatile uint32_t hookParameter4 = 0;

volatile int counter = 0;
static struct ptptime_t timestamp;

#define STP_TRACE_D32(_pstmbase_,_channel_,_stimuliport_,_type_,_data_)   ((*(volatile unsigned int   *)(_pstmbase_+0x1000*_channel_+_stimuliport_*0x100+_type_))=(unsigned int  )(_data_))

#pragma GCC push_options
#pragma GCC optimize ("O1")
void __attribute__ ((noinline)) traceHook(uint32_t hookN, uint32_t hookP1, uint32_t hookP2){
	OS_CPU_SR cpu_sr;
	OS_ENTER_CRITICAL();
	//ETH_PTPTime_GetTime(&timestamp);
	hookName = hookN;
	hookParameter1 = hookP1;
	hookParameter2 = hookP2;
	//hookParameter3 = timestamp.tv_sec;
//	hookParameter4 = timestamp.tv_nsec;
//	Xil_Out32(0xf8000000, hookN);
//	Xil_Out32(0xf8001000, hookP1);
//	Xil_Out32(0xf8002000, hookP2);
//	Xil_Out32(0xf8003000, timestamp.tv_sec);
//	Xil_Out32(0xf8004000, timestamp.tv_nsec);
	OS_EXIT_CRITICAL();
}
#pragma GCC pop_options

void imw_writer_start_hook(IMW_WRITER_CONFIG_t *p_arg) {
	traceHook(0x00000002, p_arg->object->objectId, p_arg->internal->currentObjectSeqNr);
	return;
}

void imw_writer_end_hook(IMW_WRITER_CONFIG_t *p_arg) {
	traceHook(0x00000003, p_arg->object->objectId, p_arg->internal->currentObjectSeqNr);
	counter++;
#if BOARD == 1
	if(counter == 5000){
		stopTest();
	}
#else
	if(counter == 10000){
		stopTest();
	}
#endif
	return;
}

uint32_t writerSeqNr = 0;

void ILM_HOOK_TACT(ILM_LET_TABLE_T *pEntry){
	if(!running)
		running = 1;
#if BOARD==1
	if(pEntry->TaskId == IMW_WRITER_TASK_1_ID){
#else
	if(pEntry->TaskId == IMW_WRITER_TASK_2_ID){
#endif
		traceHook(0x00000004, pEntry->TaskId, writerSeqNr);
		writerSeqNr++;
	}
	return;
}

void ILM_HOOK_TINCR(ILM_LET_TABLE_T *pEntry){
	traceHook(0x00000005, *((uint16*)(pEntry->Payload.pSchedEvent))-1, 0);
}

void imw_reader_sumbessageData_start_hook(IMW_SUBMESSAGE_DATA_HEADER_t *header) {
	traceHook(0x00000006, header->objectSequenceNr, header->fragmentId);
	return;
}

void imw_reader_sumbessageData_end_hook(IMW_SUBMESSAGE_DATA_HEADER_t *header) {
	traceHook(0x00000007, header->objectSequenceNr, header->fragmentId);
	return;
}

void imw_reader_bitmapCheckFailed_hook(IMW_READER_CONFIG_t *p_arg) {
	traceHook(0x00000008, 0, 0);
	return;
}

void emacps_tx_hook(struct pbuf* p) {
	struct eth_hdr *ethhdr;
	struct ip_hdr *iphdr;
	IMW_SUBMESSAGE_DATA_HEADER_t *imw_data_header;

	if(p->tot_len == 1086){
		ethhdr = (struct eth_hdr *)p->payload;
		if(ethhdr->type == PP_HTONS(ETHTYPE_VLAN)){
			if(p->len < p->tot_len && p->next != (struct pbuf*)0){
				imw_data_header = (IMW_SUBMESSAGE_DATA_HEADER_t*)(p->next->payload + sizeof(IMW_HEADER_t));
				traceHook(0x00000009, imw_data_header->objectSequenceNr, imw_data_header->fragmentId);
			} else {
				iphdr = (struct ip_hdr*)(p->payload + SIZEOF_ETH_HDR + SIZEOF_VLAN_HDR);
				uint32_t len = lwip_ntohs(IPH_LEN(iphdr));
				uint32_t iphdr_hlen = IPH_HL_BYTES(iphdr);
				if(len == 1068){
					imw_data_header = (IMW_SUBMESSAGE_DATA_HEADER_t*)(p->payload + SIZEOF_ETH_HDR + SIZEOF_VLAN_HDR + iphdr_hlen + UDP_HLEN + sizeof(IMW_HEADER_t));
					traceHook(0x00000009, imw_data_header->objectSequenceNr, imw_data_header->fragmentId);
				}
			}
		}
	}
	return;
}

void emacps_tx_complete_hook(struct pbuf* p) {
	struct eth_hdr *ethhdr;
	struct ip_hdr *iphdr;
	IMW_SUBMESSAGE_DATA_HEADER_t *imw_data_header;
	if(p->tot_len == 1086){
		ethhdr = (struct eth_hdr *)p->payload;
		if(ethhdr->type == PP_HTONS(ETHTYPE_VLAN)){
			if(p->len < p->tot_len && p->next != (struct pbuf*)0){
				imw_data_header = (IMW_SUBMESSAGE_DATA_HEADER_t*)(p->next->payload + sizeof(IMW_HEADER_t));
				traceHook(0x0000000A, imw_data_header->objectSequenceNr, imw_data_header->fragmentId);
			} else {
				iphdr = (struct ip_hdr*)(p->payload + SIZEOF_ETH_HDR + SIZEOF_VLAN_HDR);
				uint32_t len = lwip_ntohs(IPH_LEN(iphdr));
				uint32_t iphdr_hlen = IPH_HL_BYTES(iphdr);
				if(len == 1068){
					imw_data_header = (IMW_SUBMESSAGE_DATA_HEADER_t*)(p->payload + SIZEOF_ETH_HDR + SIZEOF_VLAN_HDR + iphdr_hlen + UDP_HLEN + sizeof(IMW_HEADER_t));
					traceHook(0x0000000A, imw_data_header->objectSequenceNr, imw_data_header->fragmentId);
				}
			}
		}
	}
	return;
}

void emacps_rx_hook(struct pbuf* p) {
	struct eth_hdr *ethhdr;
	struct ip_hdr *iphdr;
	IMW_SUBMESSAGE_DATA_HEADER_t *imw_data_header;
	if(p->tot_len == 1086){
		ethhdr = (struct eth_hdr *)p->payload;
		if(ethhdr->type == PP_HTONS(ETHTYPE_VLAN)){
			iphdr = (struct ip_hdr*)(p->payload + SIZEOF_ETH_HDR + SIZEOF_VLAN_HDR);
			uint32_t len = lwip_ntohs(IPH_LEN(iphdr));
			uint32_t iphdr_hlen = IPH_HL_BYTES(iphdr);
			if(len == 1068){
				imw_data_header = (IMW_SUBMESSAGE_DATA_HEADER_t*)(p->payload + SIZEOF_ETH_HDR + SIZEOF_VLAN_HDR + iphdr_hlen + UDP_HLEN + sizeof(IMW_HEADER_t));
				traceHook(0x0000000B, imw_data_header->objectSequenceNr, imw_data_header->fragmentId);
			}
		}
	}
	return;
}

void tcpReadHook(int tcpIteration, size_t bytesLeft){
	//traceHook(0x0000000C, (uint32_t)tcpIteration, (uint32_t)bytesLeft);
}

void tcpWriteHook(int tcpIteration, size_t bytesLeft){
	//traceHook(0x0000000D, (uint32_t)tcpIteration, (uint32_t)bytesLeft);
}

void imw_writer_send_fragment_hook(uint16_t seqNr, int fragment) {
	//traceHook(0x0000000E, seqNr, fragment);
	return;
}

void ida_lwip_socket_sendto_hook(void *data, size_t size) {
	(void) data;
	return;
}

void ida_filter_thread_input_hook(struct pbuf* p) {
	//traceHook(0x00001000, (uint32_t)p, p->tot_len);
	return;
}

void ida_filter_thread_notforus_hook(struct pbuf* p) {
	//traceHook(0x00001001, (uint32_t)p, p->tot_len);
	return;
}

void pbuf_free_hook(struct pbuf* p) {
	//traceHook(0x00001002, (uint32_t)p, p->tot_len);
	return;
}

void ida_filter_tx_thread_input_hook(IDA_LWIP_TX_REQ *txReq) {
	//traceHook(0x00002000, (uint32_t)txReq->type, (uint32_t)txReq->size);
	return;
}

void ida_filter_tx_thread_done_hook(IDA_LWIP_TX_REQ *txReq) {
	//traceHook(0x00002001, (uint32_t)txReq->type, (uint32_t)txReq->err);
	return;
}



void errorHook(uint32_t err1, uint32_t err2){
	traceHook(0x000000FF, err1, err2);
}

void ETH_PTP_SET_INCR_HOOK(uint32_t ns, uint32_t sub_ns){
	traceHook(0x01000000, ns, sub_ns);
}

/*
*********************************************************************************************************
*********************************************************************************************************
**                                         uC/OS-II APP HOOKS
*********************************************************************************************************
*********************************************************************************************************
*/

#if (OS_APP_HOOKS_EN > 0)

/*
*********************************************************************************************************
*                                      TASK CREATION HOOK (APPLICATION)
*
* Description : This function is called when a task is created.
*
* Argument(s) : ptcb   is a pointer to the task control block of the task being created.
*
* Note(s)     : (1) Interrupts are disabled during this call.
*********************************************************************************************************
*/

void  App_TaskCreateHook (OS_TCB *ptcb)
{
#if (APP_CFG_PROBE_OS_PLUGIN_EN == DEF_ENABLED) && (OS_PROBE_HOOKS_EN > 0)
    OSProbe_TaskCreateHook(ptcb);
#endif
}

/*
*********************************************************************************************************
*                                    TASK DELETION HOOK (APPLICATION)
*
* Description : This function is called when a task is deleted.
*
* Argument(s) : ptcb   is a pointer to the task control block of the task being deleted.
*
* Note(s)     : (1) Interrupts are disabled during this call.
*********************************************************************************************************
*/

void  App_TaskDelHook (OS_TCB *ptcb)
{
    (void)ptcb;
}

/*
*********************************************************************************************************
*                                      IDLE TASK HOOK (APPLICATION)
*
* Description : This function is called by OSTaskIdleHook(), which is called by the idle task.  This hook
*               has been added to allow you to do such things as STOP the CPU to conserve power.
*
* Argument(s) : none.
*
* Note(s)     : (1) Interrupts are enabled during this call.
*********************************************************************************************************
*/

#if OS_VERSION >= 251
void  App_TaskIdleHook (void)
{
	CPU_SR cpu_sr;
	OS_ENTER_CRITICAL();
	if(running)
		traceHook(0x00000000, 0, 0);
#ifdef WFI_IDLE
    __asm volatile( "dsb" );
    __asm volatile( "wfi" );
    __asm volatile( "isb" );
#endif
	if(running)
		traceHook(0x00000001, 0, 0);
    OS_EXIT_CRITICAL();
}
#endif

/*
*********************************************************************************************************
*                                        STATISTIC TASK HOOK (APPLICATION)
*
* Description : This function is called by OSTaskStatHook(), which is called every second by uC/OS-II's
*               statistics task.  This allows your application to add functionality to the statistics task.
*
* Argument(s) : none.
*********************************************************************************************************
*/

void  App_TaskStatHook (void)
{
}

/*
*********************************************************************************************************
*                                            TASK RETURN HOOK (APPLICATION)
*
* Description: This function is called if a task accidentally returns.  In other words, a task should
*              either be an infinite loop or delete itself when done.
*
* Arguments  : ptcb      is a pointer to the task control block of the task that is returning.
*
* Note(s)    : none
*********************************************************************************************************
*/


#if OS_VERSION >= 289
void  App_TaskReturnHook (OS_TCB  *ptcb)
{
    (void)ptcb;
}
#endif

/*
*********************************************************************************************************
*                                        TASK SWITCH HOOK (APPLICATION)
*
* Description : This function is called when a task switch is performed.  This allows you to perform other
*               operations during a context switch.
*
* Argument(s) : none.
*
* Note(s)     : (1) Interrupts are disabled during this call.
*
*               (2) It is assumed that the global pointer 'OSTCBHighRdy' points to the TCB of the task that
*                   will be 'switched in' (i.e. the highest priority task) and, 'OSTCBCur' points to the
*                  task being switched out (i.e. the preempted task).
*********************************************************************************************************
*/

#if OS_TASK_SW_HOOK_EN > 0
void  App_TaskSwHook (void)
{
#if (APP_CFG_PROBE_OS_PLUGIN_EN > 0) && (OS_PROBE_HOOKS_EN > 0)
    OSProbe_TaskSwHook();
#endif
    traceHook(0x000000FE, OSTCBCur->OSTCBPrio, OSTCBHighRdy->OSTCBPrio);
}
#endif

/*
*********************************************************************************************************
*                                     OS_TCBInit() HOOK (APPLICATION)
*
* Description : This function is called by OSTCBInitHook(), which is called by OS_TCBInit() after setting
*               up most of the TCB.
*
* Argument(s) : ptcb    is a pointer to the TCB of the task being created.
*
* Note(s)     : (1) Interrupts may or may not be ENABLED during this call.
*********************************************************************************************************
*/

#if OS_VERSION >= 204
void  App_TCBInitHook (OS_TCB *ptcb)
{
    (void)ptcb;
}
#endif

/*
*********************************************************************************************************
*                                        TICK HOOK (APPLICATION)
*
* Description : This function is called every tick.
*
* Argument(s) : none.
*
* Note(s)     : (1) Interrupts may or may not be ENABLED during this call.
*********************************************************************************************************
*/

#if OS_TIME_TICK_HOOK_EN > 0
void  App_TimeTickHook (void)
{
#if (APP_CFG_PROBE_OS_PLUGIN_EN == DEF_ENABLED) && (OS_PROBE_HOOKS_EN > 0)
    OSProbe_TickHook();
#endif
}
#endif
#endif
