/*
 * app.c
 *
 *  Created on: Aug 12, 2020
 *      Author: kaige
 */

#include <stdio.h>
#include "cpu_core.h"
#include "ucos_ii.h"
#include "ucos_bsp.h"

#include "netif/xemacpsif.h"
#include "netif/xadapter.h"
#include "lwip/init.h"
#include "lwip/inet.h"
#include "lwip/opt.h"
#include "lwip/ip_addr.h"


#include "lwipopts.h"
#include <ucos_int.h>
#include "lwip/sockets.h"

#include "xipipsu.h"
#include "xscugic.h"

#include "netif/xemacps_ieee1588.h"
#include "ida-lwip/ida_lwip_init.h"
#include "ida-lwip/ida_lwip_filter.h"

#include "ILM/if/ilm_core.h"
#include "xil_mpu.h"

#include "imw_reader.h"
#include "imw_writer.h"
#include "imw_sync.h"

#include "ptpd.h"

#include "imw_example_configs.h"

extern struct netif EthDev;


#define TICKS_PER_S 			10000	//10kHz
#define TICKS_PER_MACRO_PERIOD 	100	//10Hz PWM, 20ms

extern const ILM_LET_TABLE_T CO_LET_TABLE[];
extern const ILM_HW_LET_TABLE_T C0_HW_Master_Table[];
extern const size_t CO_LET_TABLE_SIZE;
extern const size_t C0_HW_TABLE_SIZE;

static CPU_STK imwWriterTask2Stk[IMW_WRITER_TASK_2_STACK_SIZE];
static CPU_STK imwRxTask2Stk[IMW_RX_TASK_2_STACK_SIZE];

IMW_OBJECT_t imwObject1 = {
		.objectId=IMW_OBJECT_1_ID,
		.size=IMW_OBJECT_1_SIZE,
		.historyCount=2,
		.baseAddr=(void*)IMW_OBJECT_1_BASE
};

IMW_OBJECT_t imwObject2 = {
		.objectId=IMW_OBJECT_2_ID,
		.size=IMW_OBJECT_2_SIZE,
		.historyCount=2,
		.baseAddr=(void*)IMW_OBJECT_2_BASE
};

uint8_t txBuffer[1500] __attribute__ ((aligned (8)));
uint8_t rxBuffer[1500] __attribute__ ((aligned (8)));

struct sockaddr_in board1 = {
		.sin_family = AF_INET,
		.sin_addr = IPADDR4_INIT_BYTES(192,168,178,10),
		.sin_port = PP_HTONS(1000)
};

struct sockaddr_in board2 = {
		.sin_family = AF_INET,
		.sin_addr = IPADDR4_INIT_BYTES(192,168,178,11),
		.sin_port = PP_HTONS(1000)
};

IMW_READER_INTERNAL_t imwReader1Internal;

IMW_READER_CONFIG_t imwReader1Config = {
		.object = &imwObject1,
		.seqNrOffset = 0,
		.fragmentSize = 1024,
		.padding = 2,
		.priority = 5,
		.maxRetry = 1,
		.target = (void*)&board1,
		.localPort = 1000,
		.rxBuffer = (void*)rxBuffer,
		.txBuffer = (void*)txBuffer,
		.internal = &imwReader1Internal
};

IMW_RX_TASK_INTERNAL_t imwRxTaskConfigInternal;

IMW_RX_TASK_CONFIG_t imwRxTaskConfig = {
		.internal = &imwRxTaskConfigInternal,
		.ready = 0,
		.taskStack = (void*)imwRxTask2Stk,
		.taskStackSize = IMW_RX_TASK_2_STACK_SIZE,
		.taskPrio = IMW_RX_TASK_2_PRIO,
		.rxBuffer = (void*)rxBuffer,
		.handleSubmessageData = imw_reader_handleSubmessageData,
		.handleSubmessageBitmap = imw_writer_handleSubmessageBitmap,
};

IMW_BITMAP_t writer2Bitmap;
IMW_WRITER_INTERNAL_t writer2ConfigInternal;

IMW_WRITER_CONFIG_t writer2Config = {
		.object=&imwObject2,
		.seqNrOffset = 1,
		.fragmentSize=1024,
		.padding=2,
		.target=(void*)&board1,
		.localPort = 2000,
		.bitmap = &writer2Bitmap,
		.txBuffer=(void*)&txBuffer,
		.rxBuffer=(void*)&rxBuffer,
		.internal=&writer2ConfigInternal,
		.period = 50,
		.taskStack = (void*)imwWriterTask2Stk,
		.taskStackSize = IMW_WRITER_TASK_2_STACK_SIZE,
		.taskPrio = IMW_WRITER_TASK_2_PRIO,
		.taskId = IMW_WRITER_TASK_2_ID,
		.rxTaskConfig = &imwRxTaskConfig,
};

void startAPP(void){
	Xil_SetMPURegion(0x800000, 0x800000, NORM_SHARED_NCACHE | PRIV_RW_USER_RW);

	ILM_HW_Set_Interface((struct xemac_s *) EthDev.state);
	ILM_Init((ILM_LET_TABLE_T *) CO_LET_TABLE, CO_LET_TABLE_SIZE,TICKS_PER_MACRO_PERIOD);
	ILM_HW_Init((ILM_HW_LET_TABLE_T *) C0_HW_Master_Table, C0_HW_TABLE_SIZE,TICKS_PER_S, TICKS_PER_MACRO_PERIOD);


	imw_reader_Init();

	imw_sync_init();

	imw_net_rx_task_init(&imwRxTaskConfig);

	imw_writer_create(&writer2Config);

	int sockfd = imw_reader_create(&imwReader1Config);
	imw_net_rx_register_handler(&imwRxTaskConfig,sockfd,(void*)&imwReader1Config);

	while (!ptpd_is_in_sync()) {
		OSTimeDly(OS_TICKS_PER_SEC/100);
	}

	ip_addr_t ipaddrBoard1 = IPADDR4_INIT_BYTES(192,168,178,10);
	TimeInternal startTime;
	imw_sync(&ipaddrBoard1, 0, &startTime);

	ILM_HW_StartTimerAt(startTime.seconds);

//	ILM_HW_StartTimer();

	

	return;
}
