/*
 * app.c
 *
 *  Created on: Aug 12, 2020
 *      Author: kaige
 */

#include <stdio.h>
#include "ucos_ii.h"
#include "ucos_bsp.h"
#include "xipipsu.h"
#include "xscugic.h"

#include <ucos_int.h>

#include "lwip/tcpip.h"
#include "lwip/dhcp.h"

#include "lwip/init.h"
#include "lwip/inet.h"
#include "lwip/opt.h"
#include "lwip/ip_addr.h"
#include "lwipopts.h"
#include "xil_mmu.h"

#include "lwip/sockets.h"

#include "ILM/if/ilm_core.h"
#include "ILM/if/ilm_os_port.h"

#include "xil_cache.h"

#define TICKS_PER_S 			10000	//10kHz
#define TICKS_PER_MACRO_PERIOD 	100	//10Hz PWM, 20ms

extern const ILM_LET_TABLE_T CO_LET_TABLE[];
extern const ILM_HW_LET_TABLE_T C0_HW_Master_Table[];
extern const size_t CO_LET_TABLE_SIZE;
extern const size_t C0_HW_TABLE_SIZE;

#include "imw_data_writer.h"
#include "imw_data_reader.h"
#include "imw_reader.h"
#include "imw_writer.h"
#include "imw_sync.h"

#include "ptpd.h"

#include "imw_example_configs.h"

extern struct netif EthDev;


static CPU_STK imwLetTask1Stk[IMW_LET_TASK_1_STACK_SIZE];
static CPU_STK imwLetTask2Stk[IMW_LET_TASK_2_STACK_SIZE];
static CPU_STK imwWriterTask1Stk[IMW_WRITER_TASK_1_STACK_SIZE];
static CPU_STK imwRxTask2Stk[IMW_RX_TASK_2_STACK_SIZE];

IMW_ERR_t letTask2Function(void* p_arg);
IMW_ERR_t letTask1Function(void* p_arg);

IMW_OBJECT_t imwObject1 = {
		.objectId=IMW_OBJECT_1_ID,
		.size=IMW_OBJECT_1_SIZE,
		.historyCount=4,
		.baseAddr=(void*)IMW_OBJECT_1_BASE
};

IMW_OBJECT_t imwObject2 = {
		.objectId=IMW_OBJECT_2_ID,
		.size=IMW_OBJECT_2_SIZE,
		.historyCount=2,
		.baseAddr=(void*)IMW_OBJECT_2_BASE
};

struct sockaddr_in board1 = {
		.sin_family = AF_INET,
		.sin_addr = IPADDR4_INIT_BYTES(192,168,178,10),
		.sin_port = PP_HTONS(1000)
};

struct sockaddr_in board2 = {
		.sin_family = AF_INET,
		.sin_addr = IPADDR4_INIT_BYTES(192,168,178,11),
		.sin_port = PP_HTONS(1000)
};

IMW_DATA_WRITER_INTERNAL_t imwDataWriter1Internal;

IMW_DATA_WRITER_CONFIG_t imwDataWriter1Config = {
		.object = &imwObject1,
		.seqNrOffset = 0,
		.taskStack = (void*)imwLetTask1Stk,
		.taskPrio = IMW_LET_TASK_1_PRIO,
		.taskStackSize = IMW_LET_TASK_1_STACK_SIZE,
		.taskId =IMW_LET_TASK_1_ID,
		.function = letTask1Function,
		.internal = &imwDataWriter1Internal,
};

IMW_DATA_READER_INTERNAL_t imwDataReader2Internal;

IMW_DATA_READER_CONFIG_t imwDataReader2Config = {
		.taskStack = (void*)imwLetTask2Stk,
		.taskPrio = IMW_LET_TASK_2_PRIO,
		.taskStackSize = IMW_LET_TASK_2_STACK_SIZE,
		.taskId =IMW_LET_TASK_2_ID,
		.function = letTask2Function,
		.internal = &imwDataReader2Internal,
};

extern void send_hook(uint32_t id);
IMW_ERR_t letTask1Function(void* p_arg){
	IMW_OBJECT_t *object;
	IMW_DATA_WRITER_INTERNAL_t *internal;
	IMW_DATA_WRITER_CONFIG_t *config = (IMW_DATA_WRITER_CONFIG_t*)p_arg;

	IMW_ARG_NULL_CHECK(config);
	IMW_ARG_NULL_CHECK(config->object);
	IMW_ARG_NULL_CHECK(config->internal);
	IMW_ARG_NULL_CHECK(config->function);

	object = config->object;
	internal = config->internal;

	void *historyBufferEntry = object->baseAddr + ((internal->currentObjectSeqNr % object->historyCount) * object->size);

	memset(historyBufferEntry, (uint8_t)internal->currentObjectSeqNr, object->size);
	send_hook(IMW_WRITER_TASK_1_ID);
	ILM_OS_ActivateTask(IMW_WRITER_TASK_1_ID);

	return IMW_ERR_NONE;
}

const uint8_t dataReader2Mapping[] = {1,1,2,2,3,3,0,0};
const uint8_t dataReader2Mapping2[] = {1,0};

#define DEBUG_ARRAY_SIZE 	1000
uint8_t debugArray[DEBUG_ARRAY_SIZE];

IMW_ERR_t letTask2Function(void* p_arg){
	IMW_OBJECT_t *object;
	IMW_DATA_READER_INTERNAL_t *internal;
	IMW_DATA_READER_CONFIG_t *config = (IMW_DATA_READER_CONFIG_t*)p_arg;

	IMW_ARG_NULL_CHECK(config);
	IMW_ARG_NULL_CHECK(config->internal);

	internal = config->internal;

	uint8_t *historyBufferEntry1 = (uint8_t*)(imwObject1.baseAddr + ((dataReader2Mapping[internal->currentSeqNr % (sizeof(dataReader2Mapping)/sizeof(uint8_t))]) * imwObject1.size));

	uint8_t *historyBufferEntry2 = (imwObject2.baseAddr + ((dataReader2Mapping2[internal->currentSeqNr % (sizeof(dataReader2Mapping2)/sizeof(uint8_t))]) * imwObject2.size));



	debugArray[internal->currentSeqNr % DEBUG_ARRAY_SIZE] = historyBufferEntry1[50] - historyBufferEntry2[50];

	return IMW_ERR_NONE;
}

uint8_t txBuffer[1500] __attribute__ ((aligned (8)));
uint8_t rxBuffer[1500] __attribute__ ((aligned (8)));

IMW_READER_INTERNAL_t imwReader2Internal;

IMW_READER_CONFIG_t imwReader2Config = {
		.object = &imwObject2,
		.seqNrOffset = 0,
		.fragmentSize = 1024,
		.padding = 2,
		.priority = 5,
		.maxRetry = 1,
		.target = (void*)&board2,
		.localPort = 1000,
		.rxBuffer = (void*)rxBuffer,
		.txBuffer = (void*)txBuffer,
		.internal = &imwReader2Internal
};

IMW_RX_TASK_INTERNAL_t imwRxTaskConfigInternal;

IMW_RX_TASK_CONFIG_t imwRxTaskConfig = {
		.internal = &imwRxTaskConfigInternal,
		.ready = 0,
		.taskStack = (void*)imwRxTask2Stk,
		.taskStackSize = IMW_RX_TASK_2_STACK_SIZE,
		.taskPrio = IMW_RX_TASK_2_PRIO,
		.rxBuffer = (void*)rxBuffer,
		.handleSubmessageData = imw_reader_handleSubmessageData,
		.handleSubmessageBitmap = imw_writer_handleSubmessageBitmap,
};

IMW_BITMAP_t writer1Bitmap;
IMW_WRITER_INTERNAL_t writer1ConfigInternal;

IMW_WRITER_CONFIG_t writer1Config = {
		.object=&imwObject1,
		.seqNrOffset = 3,
		.fragmentSize=1024,
		.padding=2,
		.target=(void*)&board2,
		.localPort = 2000,
		.bitmap = &writer1Bitmap,
		.txBuffer=(void*)&txBuffer,
		.rxBuffer=(void*)&rxBuffer,
		.internal=&writer1ConfigInternal,
		.period = 50,
		.taskStack = (void*)imwWriterTask1Stk,
		.taskStackSize = IMW_WRITER_TASK_1_STACK_SIZE,
		.taskPrio = IMW_WRITER_TASK_1_PRIO,
		.taskId = IMW_WRITER_TASK_1_ID,
		.rxTaskConfig = &imwRxTaskConfig,
};

extern void ILM_OS_RegisterActSem(uint8_t TaskId, OS_EVENT* actSem);

#define NR_WAYS   	4
#define NR_SETS  	128
#define LINE_SIZE 	(64/4)

uint32_t cachableObject[32/4*1024];
void accessCache(){
	int set = 0;
	int way = 0;
	for(set = 0; set < NR_SETS; set++){
		for(way = 0; way < NR_WAYS; way++){
			Xil_DCacheFlushLine(&cachableObject[set * way * LINE_SIZE]);
		}
	}
}

#define __burn_wcet(x,y)                \
    ({ CPU_INT32U __mul = (x);                        \
       CPU_INT32U __scale = (y);                      \
       asm volatile(	"		mov x0,#0		\n\t" \
						"		mul x1,%0,%1	\n\t" \
    					"1: 	add x0,x0,#1	\n\t" \
    					"		cmp x1,x0		\n\t" \
    					" 		bne 1b 			\n\t" \
    		   	   : : "r" (__mul) , "r" (__scale) : ); })

extern void traceHook(uint32_t hookN, uint32_t hookP1, uint32_t hookP2);

static OS_STK dummyLoadTaskStack[1024];
static void dummyLoadTask(void *p_arg) {
	uint8_t err;
	OS_EVENT *sem;

	/* Initialize IPI IRQ for activation */
	sem = OSSemCreate(0);
	ILM_OS_RegisterActSem(9, sem);

	while(1){
		OSSemPend(sem,0,&err);
		uint32_t wcet = 300 + rand() % (400+1 - 300);
		traceHook(0xABCDE, 1 , wcet);
		__burn_wcet(wcet,600);
		traceHook(0xABCDE, 2 , wcet);
		accessCache();
		traceHook(0xABCDE, 3 , wcet);
	}
}

extern void tcpTokenRingInit(void);

void startAPP(void){
	uint8_t err = 0;
	Xil_SetTlbAttributes(0x800000, NORM_NONCACHE);
	
	ILM_HW_Set_Interface((struct xemac_s *) EthDev.state);
	ILM_Init((ILM_LET_TABLE_T *) CO_LET_TABLE, CO_LET_TABLE_SIZE,TICKS_PER_MACRO_PERIOD);
	ILM_HW_Init((ILM_HW_LET_TABLE_T *) C0_HW_Master_Table, C0_HW_TABLE_SIZE,TICKS_PER_S, TICKS_PER_MACRO_PERIOD);
	
	err = OSTaskCreateExt( dummyLoadTask,
				(void*)0,
				&dummyLoadTaskStack[1024-1],
				1,
				1,
				dummyLoadTaskStack,
				1024,
				 DEF_NULL,
				(OS_TASK_OPT_STK_CLR | OS_TASK_OPT_STK_CHK));

	OSTaskNameSet(1,(INT8U *)"dummyLoadTask",NULL);

	tcpTokenRingInit();
	OSTimeDly(10*OS_TICKS_PER_SEC);

	memset(debugArray,0xAB,sizeof(debugArray));

	imw_dataWriter_create(&imwDataWriter1Config);
	imw_dataReader_create(&imwDataReader2Config);
	imw_reader_Init();

	imw_sync_init();

	imw_net_rx_task_init(&imwRxTaskConfig);

	imw_writer_create(&writer1Config);

	int sockfd = imw_reader_create(&imwReader2Config);
	imw_net_rx_register_handler(&imwRxTaskConfig,sockfd,(void*)&imwReader2Config);

	while (!ptpd_is_in_sync()) {
		OSTimeDly(OS_TICKS_PER_SEC/100);
	}

	ip_addr_t ipaddrBoard2 = IPADDR4_INIT_BYTES(192,168,178,11);
	TimeInternal startTime;
	imw_sync(&ipaddrBoard2, 1, &startTime);

	ILM_HW_StartTimerAt(startTime.seconds);

//	ILM_HW_StartTimer();

	return;
}
