/*
 * imw_data_reader.c
 *
 *  Created on: May 25, 2020
 *      Author: kaige
 */

#include "imw_data_reader.h"
#include "ucos_ii.h"

static void _imwDataReaderTask(void *p_arg);

IMW_ERR_t imw_dataReader_create(IMW_DATA_READER_CONFIG_t *config){
	IMW_OBJECT_t *object;
	IMW_DATA_READER_INTERNAL_t *internal;
	uint8_t err = 0;

	IMW_ARG_NULL_CHECK(config);
	IMW_ARG_NULL_CHECK(config->internal);

	internal = config->internal;

	internal->currentSeqNr = 0;

	err = OSTaskCreateExt( _imwDataReaderTask,
				(void*)config,
				&((OS_STK*)(config->taskStack))[config->taskStackSize -1],
				config->taskPrio,
				config->taskId,
				(OS_STK*)(config->taskStack),
				config->taskStackSize,
				 DEF_NULL,
				(OS_TASK_OPT_STK_CLR | OS_TASK_OPT_STK_CHK));
	if(err != OS_ERR_NONE)
			return IMW_ERR_OS;

	OSTaskNameSet(config->taskPrio,(INT8U *)"imwDataReader",(void*)0);

	return IMW_ERR_NONE;
}

extern void ILM_OS_RegisterActSem(uint8_t TaskId, OS_EVENT* actSem);

static void _imwDataReaderTask(void *p_arg) {
	uint8_t err;
	IMW_DATA_READER_CONFIG_t *config = (IMW_DATA_READER_CONFIG_t *)p_arg;
	IMW_DATA_READER_INTERNAL_t *internal = config->internal;
	OS_EVENT *sem;

	/* Initialize IPI IRQ for activation */
	sem = OSSemCreate(0);
	ILM_OS_RegisterActSem(config->taskId, sem);

	while(1){
		OSSemPend(sem,0,&err);
		if(config->function(p_arg) != IMW_ERR_NONE)
			OSTaskDel(OS_PRIO_SELF);
		internal->currentSeqNr++;
	}
}
