/*
 * imw_data_writer.c
 *
 *  Created on: May 5, 2020
 *      Author: kaige
 */


#include "imw_data_writer.h"
#include "string.h"		/* for memcpy */
#include "ucos_ii.h"

static void _imwDataWriterTask(void *p_arg);

IMW_ERR_t imw_dataWriter_create(IMW_DATA_WRITER_CONFIG_t *config){
	IMW_OBJECT_t *object;
	IMW_DATA_WRITER_INTERNAL_t *internal;
	uint8_t err = 0;

	IMW_ARG_NULL_CHECK(config);
	IMW_ARG_NULL_CHECK(config->internal);

	internal = config->internal;

	internal->currentObjectSeqNr = config->seqNrOffset;

	err = OSTaskCreateExt( _imwDataWriterTask,
				(void*)config,
				&((OS_STK*)(config->taskStack))[config->taskStackSize -1],
				config->taskPrio,
				config->taskId,
				(OS_STK*)(config->taskStack),
				config->taskStackSize,
				 DEF_NULL,
				(OS_TASK_OPT_STK_CLR | OS_TASK_OPT_STK_CHK));
	if(err != OS_ERR_NONE)
			return IMW_ERR_OS;

	OSTaskNameSet(config->taskPrio,(INT8U *)"imwDataWriter",NULL);

	return IMW_ERR_NONE;
}

/**
 * Write data to the current object entry
 */
IMW_ERR_t imw_dataWriter_write(IMW_DATA_WRITER_CONFIG_t *config, void*data, uint32_t len){
	IMW_OBJECT_t *object;
	IMW_DATA_WRITER_INTERNAL_t *internal;

	IMW_ARG_NULL_CHECK(config);
	IMW_ARG_NULL_CHECK(data);
	IMW_ARG_NULL_CHECK(config->object);
	IMW_ARG_NULL_CHECK(config->internal);

	object = config->object;
	internal = config->internal;

	if(len > object->size)
		return IMW_ERR_ARG;

	void *historyBufferEntry = object->baseAddr + ((internal->currentObjectSeqNr % object->historyCount) * object->size);
	memcpy(historyBufferEntry, data, len);
	return IMW_ERR_NONE;
}

/**
 * \brief This function is for testing purposes.
 * It fills the object entry with the seqNr (converted to char)
 */
IMW_ERR_t imw_dataWriter_memset(void *p_arg){
	IMW_OBJECT_t *object;
	IMW_DATA_WRITER_INTERNAL_t *internal;
	IMW_DATA_WRITER_CONFIG_t *config = (IMW_DATA_WRITER_CONFIG_t*)p_arg;

	IMW_ARG_NULL_CHECK(config);
	IMW_ARG_NULL_CHECK(config->object);
	IMW_ARG_NULL_CHECK(config->internal);
	IMW_ARG_NULL_CHECK(config->function);

	object = config->object;
	internal = config->internal;

	void *historyBufferEntry = object->baseAddr + ((internal->currentObjectSeqNr % object->historyCount) * object->size);

	memset(historyBufferEntry, (uint8_t)internal->currentObjectSeqNr, object->size);

	return IMW_ERR_NONE;
}

extern void ILM_OS_RegisterActSem(uint8_t TaskId, OS_EVENT* actSem);

static void _imwDataWriterTask(void *p_arg) {
	uint8_t err;
	IMW_DATA_WRITER_CONFIG_t *config = (IMW_DATA_WRITER_CONFIG_t *)p_arg;
	IMW_DATA_WRITER_INTERNAL_t *internal = config->internal;
	OS_EVENT *sem;

	/* Initialize IPI IRQ for activation */
	sem = OSSemCreate(0);
	ILM_OS_RegisterActSem(config->taskId, sem);

	while(1){
		OSSemPend(sem,0,&err);
		if(config->function(p_arg) != IMW_ERR_NONE)
			OSTaskDel(OS_PRIO_SELF);
		internal->currentObjectSeqNr++;
	}
}

