/*
 * imw_sync.c
 *
 *  Created on: 22.09.2020
 *      Author: kaige
 */


#include "imw_sync.h"
#include "lwip/sockets.h"
#include "ucos_ii.h"

static int imw_sync_sockfd;

IMW_ERR_t imw_sync_init(void){
	struct sockaddr_in local;
	imw_sync_sockfd = -1;
/* Create Sync Socket */
	imw_sync_sockfd = socket(AF_INET, SOCK_DGRAM, 0);
	if (imw_sync_sockfd == -1) {
		return -1;
	}

	memset(&local, 0, sizeof(struct sockaddr_in));
	local.sin_family = AF_INET;
	local.sin_addr.s_addr = htonl(INADDR_ANY);
	local.sin_port = htons(IMW_SYNC_PORT);
	if(bind(imw_sync_sockfd, (struct sockaddr *)&local, sizeof(struct sockaddr_in)) == -1){
		close(imw_sync_sockfd);
		return IMW_ERR_SOCKET;
	}

#ifdef IDA_LWIP
	ida_lwip_set_socket_prio(imw_sync_sockfd,0);
#endif

	return IMW_ERR_NONE;
}

IMW_ERR_t imw_sync(ip_addr_t *nextAddress, int master, TimeInternal *time){
	uint32_t _imw_sync_buffer[2];
	struct sockaddr_in remote;
	remote.sin_family = AF_INET;
	remote.sin_addr.s_addr = nextAddress->addr;
	remote.sin_port = PP_HTONS(IMW_SYNC_PORT);

	if(master){
		_imw_sync_buffer[0] = IMW_TOKEN_INIT;
		/* Send init token */
		sendto(imw_sync_sockfd, (void*)_imw_sync_buffer, sizeof(_imw_sync_buffer),0, (const struct sockaddr*)&remote, sizeof(struct sockaddr_in));
		_imw_sync_buffer[0] = IMW_TOKEN_NONE;
		/* Wait until the init token comes back */
		recv(imw_sync_sockfd,(void*)_imw_sync_buffer, sizeof(_imw_sync_buffer),0);
		if(_imw_sync_buffer[0] != IMW_TOKEN_INIT)
			return IMW_ERR_SYNC;

		getTime(time);
		time->seconds = time->seconds + 2;
		time->nanoseconds = 0;
		_imw_sync_buffer[0] = IMW_TOKEN_SYNC;
		_imw_sync_buffer[1] = time->seconds;
		/* Send init token */
		sendto(imw_sync_sockfd, (void*)_imw_sync_buffer, sizeof(_imw_sync_buffer),0, (const struct sockaddr*)&remote, sizeof(struct sockaddr_in));
	} else {
		_imw_sync_buffer[0] = IMW_TOKEN_NONE;
		/*Receive init token */
		recv(imw_sync_sockfd,(void*)_imw_sync_buffer, sizeof(_imw_sync_buffer),0);
		/* If something went wrong, pass abort token */
		if(_imw_sync_buffer[0] != IMW_TOKEN_INIT){
			_imw_sync_buffer[0] = IMW_TOKEN_ERR;
			sendto(imw_sync_sockfd, (void*)_imw_sync_buffer, sizeof(_imw_sync_buffer),0, (const struct sockaddr*)&remote, sizeof(struct sockaddr_in));
			return IMW_ERR_SYNC;
		}
		/* Pass init token to the next one */
		sendto(imw_sync_sockfd, (void*)_imw_sync_buffer, sizeof(_imw_sync_buffer),0, (const struct sockaddr*)&remote, sizeof(struct sockaddr_in));
		/* Receive sync token */
		recv(imw_sync_sockfd,(void*)_imw_sync_buffer, sizeof(_imw_sync_buffer),0);
		if(_imw_sync_buffer[0] != IMW_TOKEN_SYNC){
			_imw_sync_buffer[0] = IMW_TOKEN_ERR;
			sendto(imw_sync_sockfd, (void*)_imw_sync_buffer, sizeof(_imw_sync_buffer),0, (const struct sockaddr*)&remote, sizeof(struct sockaddr_in));
			return IMW_ERR_SYNC;
		}
		time->seconds = _imw_sync_buffer[1];
		time->nanoseconds = 0;
		/* Pass sync token to the next one */
		sendto(imw_sync_sockfd, (void*)_imw_sync_buffer, sizeof(_imw_sync_buffer),0, (const struct sockaddr*)&remote, sizeof(struct sockaddr_in));
	}
	return IMW_ERR_NONE;
}
