/* sys.c */

#include "ptpd.h"
#include "netif/xemacps_ieee1588.h"

/**
 * Read time from running ptp timer (take a snapshot)
 */
void getTime(TimeInternal *time){

	struct ptptime_t timestamp;
	ETH_PTPTime_GetTime((struct ptptime_t*)&timestamp);
	time->seconds = timestamp.tv_sec;
	time->nanoseconds = timestamp.tv_nsec;
}


/**
 * Set new time in ptp timer register (no adjustment, just a hard reset)
 */
void setTime(const TimeInternal *time)
{
	struct ptptime_t ts;

	ts.tv_sec = time->seconds;
	ts.tv_nsec = time->nanoseconds;
	ETH_PTPTime_SetTime((struct ptptime_t*)&ts);
	DBG("resetting system clock to %d sec %d nsec\n", time->seconds, time->nanoseconds);
}

uint32_t getRand(uint32_t randMax)
{
	return rand() % randMax;
}

/**
 * Adjust frequency of ptp clock
 */
bool  adjFreq(int32_t adj)
{
	DBGV("adjFreq %d\n", adj);

	if (adj > ADJ_FREQ_MAX)
		adj = ADJ_FREQ_MAX;
	else if (adj < -ADJ_FREQ_MAX)
		adj = -ADJ_FREQ_MAX;

	/* Fine update method */
	ETH_PTPTime_AdjFreq((int32_t )adj);

	return TRUE;
}
