/* timer.c */

#include "ptpd.h"
#include "constants.h"

/* An array to hold the various system timer handles. */
static u32_t ptpdTimers[TIMER_ARRAY_SIZE];
static bool ptpdTimersExpired[TIMER_ARRAY_SIZE];
static u32_t startTime[TIMER_ARRAY_SIZE];
 
void timerCallback()
{
	u32_t current_time = sys_now();
	int32_t i;

	for(i = 0; i < TIMER_ARRAY_SIZE; i++)
	{
		if (ptpdTimers[i])
		{
			int32_t delta = current_time > startTime[i] ? current_time - startTime[i]: 0xFFFFFFFF - startTime[i] + current_time;
			if(ptpdTimers[i] < delta)
			{
				ptpdTimers[i] = 0;
				ptpdTimersExpired[i] = TRUE;
			}
		}
	}
}

void initTimer(void)
{
	int32_t i;

	DBG("initTimer\n");

	/* Create the various timers used in the system. */
	for (i = 0; i < TIMER_ARRAY_SIZE; i++)
	{
	// Mark the timer as not expired.
	// Initialize the timer.
	    ptpdTimers[i] = 0;
	    startTime[i] = 0;
	    ptpdTimersExpired[i] = FALSE;
	}
}


void timerStop(int32_t index)
{
	/* Sanity check the index. */
	if (index >= TIMER_ARRAY_SIZE) return;

	// Cancel the timer and reset the expired flag.
	DBGV("timerStop: stop timer %d\n", index);
	ptpdTimers[index] = 0;
	ptpdTimersExpired[index] = FALSE;
}

void timerStart(int32_t index, uint32_t interval_ms)
{
	/* Sanity check the index. */
	if (index >= TIMER_ARRAY_SIZE) return;

	DBGV("timerStart: set timer %d to %d\n", index, interval_ms);
	ptpdTimers[index] = interval_ms;
	startTime[index] = sys_now();

	ptpdTimersExpired[index] = FALSE;
	return;
}

bool timerExpired(int32_t index)
{
	/* Sanity check the index. */
	if (index >= TIMER_ARRAY_SIZE) return FALSE;

	/* Determine if the timer expired. */
	if (!ptpdTimersExpired[index]) return FALSE;
	DBGV("timerExpired: timer %d expired\n", index);
	ptpdTimersExpired[index] = FALSE;

	return TRUE;
}
